@@ -19,7 +19,12 @@
 static PyObject *eventLoopJobWrapper(PyObject *jobFn, PyObject *Py_UNUSED(_)) {
   PyObject *ret = PyObject_CallObject(jobFn, NULL);
   Py_XDECREF(ret); // don't care about its return value
+
+  PyObject *type, *value, *traceback;
+  PyErr_Fetch(&type, &value, &traceback); // Protects `decCounter()`. If the error indicator is set, Python cannot make further function calls.
   PyEventLoop::_locker->decCounter();
+  PyErr_Restore(type, value, traceback);
+
   if (PyErr_Occurred()) {
     return NULL;
   } else {
@@ -38,13 +43,16 @@ static PyObject *timerJobWrapper(PyObject *jobFn, PyObject *args) {
   PyEventLoop::AsyncHandle::id_t handleId = PyLong_AsLong(PyTuple_GetItem(args, 1));
   double delaySeconds = PyFloat_AsDouble(PyTuple_GetItem(args, 2));
   bool repeat = (bool)PyLong_AsLong(PyTuple_GetItem(args, 3));
-  auto handle = PyEventLoop::AsyncHandle::fromId(handleId);
 
   PyObject *ret = PyObject_CallObject(jobFn, NULL); // jobFn()
   Py_XDECREF(ret); // don't care about its return value
 
   PyObject *errType, *errValue, *traceback; // we can't call any Python code unless the error indicator is clear
   PyErr_Fetch(&errType, &errValue, &traceback);
+  // Making sure a `AsyncHandle::fromId` call is close to its `handle`'s use.
+  // We need to ensure the memory block doesn't move for reallocation before we can use the pointer,
+  // as we could have multiple new `setTimeout` calls to expand the `_timeoutIdMap` vector while running the job function in parallel.
+  auto handle = PyEventLoop::AsyncHandle::fromId(handleId);
   if (repeat && !handle->cancelled()) {
     _enqueueWithDelay(_loop, handleId, jobFn, delaySeconds, repeat);
   } else {
@@ -119,6 +127,7 @@ PyEventLoop::Future PyEventLoop::ensureFuture(PyObject *awaitable) {
   Py_DECREF(args);
   Py_DECREF(kwargs);
 
+  Py_INCREF(futureObj); // needs to be kept alive as `PyEventLoop::Future` will decrease the reference count in its destructor
   return PyEventLoop::Future(futureObj);
 }
 
@@ -131,10 +140,44 @@ PyEventLoop PyEventLoop::_loopNotFound() {
 /* static */
 PyEventLoop PyEventLoop::_getLoopOnThread(PyThreadState *tstate) {
   // Modified from Python 3.9 `get_running_loop` https://github.com/python/cpython/blob/7cb3a44/Modules/_asynciomodule.c#L241-L278
-  #if PY_VERSION_HEX >= 0x03090000 // Python version is greater than 3.9
-  PyObject *ts_dict = _PyThreadState_GetDict(tstate);  // borrowed reference
+
+  PyObject *ts_dict;
+  #if PY_VERSION_HEX >= 0x030d0000 // Python version is greater than 3.13
+  // The private `_PyThreadState_GetDict(tstate)` API gets removed in Python 3.13.
+  // However, simply replacing it with `PyThreadState_GetDict()` does not work,
+  //   since the public `PyThreadState_GetDict()` API can only get from the current thread.
+  // We need to somehow get the thread dictionary on the main thread instead of the current thread.
+  //
+  // UPDATE: We don't need the thread dictionary anymore.
+  // To get the thread's running event-loop in Python 3.13 is as simple as `thread_state->asyncio_running_loop`
+  {
+    // Every `PyThreadState` is actually allocated with extra fields as a `_PyThreadStateImpl` struct
+    // See https://github.com/python/cpython/blob/v3.13.0rc1/Include/internal/pycore_tstate.h#L17-L24
+    using PyThreadStateHolder = struct { // _PyThreadStateImpl
+      PyThreadState base;
+      #if PY_VERSION_HEX >= 0x030e0000 // Python version is greater than 3.14
+      // the struct is changed with more additional fields, see https://github.com/python/cpython/blob/v3.14.0rc3/Include/internal/pycore_tstate.h#L24-L40
+      Py_ssize_t refcount;
+      uintptr_t c_stack_top;
+      uintptr_t c_stack_soft_limit;
+      uintptr_t c_stack_hard_limit;
+      #endif
+      PyObject *asyncio_running_loop; // we only need the first few fields of `_PyThreadStateImpl`
+    };
+
+    // Modified from https://github.com/python/cpython/blob/v3.13.0rc1/Modules/_asynciomodule.c#L3205-L3210
+    PyObject *loop = ((PyThreadStateHolder *)tstate)->asyncio_running_loop;
+    if (loop == NULL) {
+      return _loopNotFound();
+    }
+
+    Py_INCREF(loop);
+    return PyEventLoop(loop);
+  }
+  #elif PY_VERSION_HEX >= 0x03090000 // Python version is greater than 3.9
+  ts_dict = _PyThreadState_GetDict(tstate);  // borrowed reference
   #else // Python 3.8
-  PyObject *ts_dict = tstate->dict; // see https://github.com/python/cpython/blob/v3.8.17/Modules/_asynciomodule.c#L244-L245
+  ts_dict = tstate->dict; // see https://github.com/python/cpython/blob/v3.8.17/Modules/_asynciomodule.c#L244-L245
   #endif
   if (ts_dict == NULL) {
     return _loopNotFound();
@@ -206,6 +249,14 @@ void PyEventLoop::AsyncHandle::cancel() {
   Py_XDECREF(ret);
 }
 
+/* static */
+bool PyEventLoop::AsyncHandle::cancelAll() {
+  for (AsyncHandle &handle: _timeoutIdMap) {
+    handle.cancel();
+  }
+  return true;
+}
+
 bool PyEventLoop::AsyncHandle::cancelled() {
   // https://docs.python.org/3/library/asyncio-eventloop.html#asyncio.Handle.cancelled
   PyObject *ret = PyObject_CallMethod(_handle, "cancelled", NULL); // returns Python bool
