@@ -273,7 +273,8 @@ def test_promises():
     assert "nested" == await pm.eval("(promise) => promise")(c())
     assert "nested" == await pm.eval("(promise) => promise")(await c())
     assert "nested" == await pm.eval("(promise) => promise")(await (await c()))
-    with pytest.raises(TypeError, match="object str can't be used in 'await' expression"):
+    with pytest.raises(TypeError, match="(object str can't be used in 'await' expression)|('str' object can't be awaited)"):
+      # Python 3.14 changed the error message format
       await pm.eval("(promise) => promise")(await (await (await c())))
 
     # Python awaitable throwing exceptions
